import subprocess
import os
import json

file_in = input("Input File Name: ")
file_out = input("Output File Name: ")
setcrf = input("Set Transcode CRF: ")
transcode2k = input("Transcode and Scale (y/n): ")
if(transcode2k == 'y'):
    scale = input("Enter Scale ([width]:[height]) ('2k' for 1080p): ")
    if(scale == '2k'):
        downscale = '-vf scale=1920:1080'
    else:
        downscale = '-vf scale=' + scale
    print("proc [info]: Downscale: Yes")
else:
    downscale = ''
    print("proc [info]: Downscale: No")
deletefiles = input("Delete Temp Files When Finished (y/n): ")

mkvtoolnix_dir = './mkvtoolnix'

probe = subprocess.Popen('ffprobe -hide_banner -loglevel quiet -select_streams v -print_format json -show_frames -read_intervals "%+#1" -show_entries "frame=color_space,color_primaries,color_transfer,side_data_list,pix_fmt" -i "' + file_in + '"', stdout=subprocess.PIPE)
output = json.loads(probe.stdout.read())

mdm = output['frames'][0]['side_data_list'][0]
min_luminance = mdm['min_luminance'].split('/')[0]
max_luminance = mdm['max_luminance'].split('/')[0]

cllm = output['frames'][0]['side_data_list'][1]
max_content = cllm['max_content']
max_average = cllm['max_average']

for object in output['frames'][0]['side_data_list']:
    if(object['side_data_type'] == "Dolby Vision RPU Data"):
        isdolby = True
        break
    if(object['side_data_type'] == "Dolby Vision Metadata"):
        isdolby = True
        break

if(isdolby):
    print("proc [info]: Dolby Vision: Yes")
else:
    print("proc [info]: Dolby Vision: No")
    
try:
    os.mkdir("tmp")
except FileExistsError:
    pass

transcode = subprocess.Popen('ffmpeg -y -i "' + file_in + '" -crf ' + setcrf + ' ' + downscale + ' -map 0:v -hide_banner -loglevel warning -pix_fmt yuv420p10le -c:v libx265 -x265-params hdr-opt=1:bframes=3:repeat-headers=1:info=0:colorprim=bt2020:transfer=smpte2084:colormatrix=bt2020nc:master-display="G(13250,34500)B(7500,3000)R(34000,16000)WP(15635,16450)L(' + str(max_luminance) + ',' + str(min_luminance) + ')":max-cll=' + str(max_content) + ',' + str(max_average) + ' "tmp/transcode.mkv"')
transcode.wait()
print("proc [info]: Transcode Complete")

video_track = 'tmp/transcode.mkv'

#Add back Dolby Vision if it exists
if(isdolby):
    print("proc [info]: Extracting Stream...")
    extract_hevc = subprocess.Popen('"' + mkvtoolnix_dir + 'mkvextract" tmp/transcode.mkv tracks 0:tmp/extract.hevc')
    extract_hevc.wait()
    print("proc [info]: Stream Extracted!")
    print("proc [info]: Extracting RPU...")
    os.system('ffmpeg -i "' + file_in + '" -c:v copy -hide_banner -loglevel warning -vbsf hevc_mp4toannexb -f hevc - | dovi_tool -m 2 --crop extract-rpu - -o tmp/rpu.bin')
    print("proc [info]: RPU Extracted!")

    print("proc [info]: Injecting RPU...")
    insert_rpu = subprocess.Popen('dovi_tool inject-rpu -i tmp/extract.hevc --rpu-in tmp/rpu.bin -o tmp/complete.hevc')
    insert_rpu.wait()
    print("proc [info]: RPU Injected!")

    video_track = 'tmp/complete.hevc'
    
print("proc [info]: Combining Files...")
combine = subprocess.Popen('"' + mkvtoolnix_dir + 'mkvmerge" -o "' + file_out + '" "' + video_track + '" --no-video "' + file_in + '"')
combine.wait()
print("proc [info]: Combination Complete!")

if(deletefiles == 'y'):
    print("proc [info]: Deleting Files...")
    os.remove('tmp/transcode.mkv')
    os.remove('tmp/extract.hevc')
    os.remove('tmp/rpu.bin')
    os.remove('tmp/complete.hevc')
    print("proc [info]: Files Deleted")

input("Press any key to exit...")

    
    



